# restClient.py

# ------------------------------------------------------------------------------
# Copyright (C) 2016 DELL EMC Corporation. All Rights Reserved.
# ------------------------------------------------------------------------------

"""
This module contains class for REST API calls.

pylint version: 1.5.6
pylint score: 10.00/10
"""

# disabling:
# C0103: invalid-name(for using camel case for variable names)
# R0904: too-many-public-methods
# pylint: disable=C0103, R0904, C0301

#import logging
import requests
from requests import codes

class RESTClient(object):
    """
    Description - Class for REST API. contains methods for all REST calls.

    """    
    def checkResponse(self, statusCode):
        """
        Description : Common function to check response code
        Parameters :
                    statusCode: response code (200, ok)
        Raise : raise exception if response code not expected
        """
        if statusCode != codes.ok:
            raise Exception('Invalid response code :', statusCode)

    def __init__(self, username=None, password=None, auth_type=None, auth_params=None):
        """
        Description:
            init for initializing class
        Parameters:
            username   -   User name to use when connecting to host(STRING)
            password   -   Password to use when connecting to host(STRING)
            auth_type   -   Authentication Type required (String) e.g "VRO_CONTROLCENTER"
            auth_params - Dictionary supplied for additional authentication parameters. Parameters are:
                            For VRO_CONTROLCENTER
                            auth_params['vroHost'] - VRO hostname
                            auth_params['vroAdminDomain'] - VRO Administrator Domain
        """

        self.auth = None
        self.cookies = None
        if username and password:
            print("REST Client username: %s" % username)
            self.auth = requests.auth.HTTPBasicAuth(username, password)

        #Default headers type, can be updated, based on provided headers value.
        self.headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        self.verify = False

    def get(self, url, headers=None, cookies=None, **kwargs):
        """
        Description:
            This method contains handler for REST GET call.
        Parameters:
            url - Complete location url path for running for REST call(STRING)
            headers - (OPTIONAL) Content-Type: application/(json/xml)
            kwargs - (OPTIONAL) parameters used in REST request.

        Returns:
            Response object
        """
        return self.execute('GET', url, headers=headers, cookies=cookies, **kwargs)

    def post(self, url, headers=None, cookies=None, **kwargs):
        """
        Description:
            This method contains handler for REST POST call.
        Parameters:
            url - Complete location url path for running for REST call(STRING)
            headers - (OPTIONAL) Content-Type: application/(json/xml)
            kwargs - (OPTIONAL) parameters used in REST request.

        Returns:
            Response object
        """

        return self.execute("POST", url, headers=headers, cookies=cookies, **kwargs)

    def put(self, url, headers=None, cookies=None, **kwargs):
        """
        Description: This method contains handler for REST PUT call.
        Parameters:  url - Complete location url path for running for REST call(STRING)
                     headers - (OPTIONAL) Content-Type: application/(json/xml)
                     kwargs - (OPTIONAL) parameters used in REST request.
        Returns:    Response object
        """
        return self.execute("PUT", url, headers=headers, cookies=cookies, **kwargs)

    def patch(self, url, headers=None, cookies=None, **kwargs):
        """
        Description: This method contains handler for REST PATCH call.
        Parameters:  url - Complete location url path for running for REST call(STRING)
                     headers - (OPTIONAL) Content-Type: application/(json/xml)
                     kwargs - (OPTIONAL) parameters used in REST request.
        Returns:    Response object
        """
        return self.execute("PATCH", url, headers=headers, cookies=cookies, **kwargs)

    def delete(self, url, headers=None, cookies=None, **kwargs):
        """
        Description: This method contains handler for REST DELETE call.
        Parameters:  url - Complete location url path for running for REST call(STRING)
                     headers - (OPTIONAL) Content-Type: application/(json/xml)
                     kwargs - (OPTIONAL) parameters used in REST request.
        Returns:    Response object
        """
        return self.execute("DELETE", url, headers=headers, cookies=cookies, **kwargs)

    def execute(self, method, url, headers=None, cookies=None, **kwargs):
        """
        Description:
            Will perform HTTP call support GET, HEAD, POST, PUT and DELETE.
        Parameters:
            method - Type of request method.
            url - Complete location url path for running for REST call(STRING)
            headers - (OPTIONAL) Content-Type: application/(json/xml)
            kwargs - (OPTIONAL) parameters used in REST request.

        Returns:
            Response object
        """
        try:
            #Update headers with new header values
            #print("%s request URL: %s", method, url)
            reqheaders = self.headers or {}
            reqheaders.update(headers or {})

            # Update cookies with new cookie values
            req_cookies = self.cookies or {}
            req_cookies.update(cookies or {})

            responsdata = requests.request(method, url, headers=reqheaders, auth=self.auth, verify=self.verify,
                                           cookies=req_cookies, **kwargs)
            return responsdata

        except Exception as ex:
            #print("Request Exception : %s", ex)
            raise

    def uploadFile(self, url, enableLog=True, cookies=None, **kwargs):
        """
        Description:
            Will perform HTTP call for POST request for uploading file.
        Parameters:
            url - Complete location url path for running for REST call(STRING)
            kwargs - (OPTIONAL) parameters used in REST request.
            enableLog - Modifying logs when required based on input parameter (BOOLEAN)

        Returns:
            Response object
        """

        try:
            if enableLog:
                print("POST request URL: %s", url)
            else:
                print("Calling POST request, url is not logged as it contains password")

            # Update cookies with new cookie values
            req_cookies = self.cookies or {}
            req_cookies.update(cookies or {})

            responsdata = requests.request("POST", url, auth=self.auth, verify=self.verify,
                                           cookies=req_cookies, **kwargs)
            return responsdata

        except Exception as ex:
            print("Request Exception : %s", ex)
            raise